use std::{collections::HashSet, mem::size_of};

use zerocopy::{AsBytes, FromBytes, FromZeroes, Unaligned};

use crate::{
    mapped::Slabable, transaction, FilePointer, FileRange, RawFilePointer, ReaderTrait,
    TransactionHandle, U64,
};

use super::vector::Vector;

#[derive(Clone, Copy, FromBytes, FromZeroes, AsBytes, Unaligned)]
#[repr(transparent)]
pub struct Str(Vector);

impl Slabable for Str {
    fn slabs(slabs: &mut HashSet<u32>) {
        slabs.insert(size_of::<Str>() as u32);
    }
}

impl Str {
    pub fn new() -> Self {
        Self(Vector::new())
    }
}

impl Default for Str {
    fn default() -> Self {
        Self::new()
    }
}

impl Str {
    pub fn set<R>(self, transaction: &mut TransactionHandle<R>, s: &str) -> Self {
        Self(self.0.set(transaction, s.as_bytes()))
    }

    pub fn get(self, reader: &impl ReaderTrait) -> &str {
        std::str::from_utf8(self.0.get(reader)).unwrap()
    }
}

impl FilePointer<Str> {
    pub fn set<R>(self, transaction: &mut TransactionHandle<R>, s: &str) -> FilePointer<Str> {
        let new_str = transaction.read::<Str>(self).set(transaction, s);
        let (ptr, data) = transaction.modify(self);
        *data = new_str;
        ptr
    }

    pub fn get(self, reader: &impl ReaderTrait) -> &str {
        reader.read(self).get(reader)
    }
}
